@@ -36,6 +36,7 @@ import (
 )
 
 var addonListOutput string
+var addonPrintDocs bool
 
 // AddonListTemplate represents the addon list template
 type AddonListTemplate struct {
@@ -58,7 +59,7 @@ var addonsListCmd = &cobra.Command{
 		}
 		switch strings.ToLower(addonListOutput) {
 		case "list":
-			printAddonsList(cc)
+			printAddonsList(cc, addonPrintDocs)
 		case "json":
 			printAddonsJSON(cc)
 		default:
@@ -68,13 +69,8 @@ var addonsListCmd = &cobra.Command{
 }
 
 func init() {
-	addonsListCmd.Flags().StringVarP(
-		&addonListOutput,
-		"output",
-		"o",
-		"list",
-		`minikube addons list --output OUTPUT. json, list`)
-
+	addonsListCmd.Flags().StringVarP(&addonListOutput, "output", "o", "list", "minikube addons list --output OUTPUT. json, list")
+	addonsListCmd.Flags().BoolVarP(&addonPrintDocs, "docs", "d", false, "If true, print web links to addons' documentation if using --output=list (default).")
 	AddonsCmd.AddCommand(addonsListCmd)
 }
 
@@ -92,39 +88,56 @@ var stringFromStatus = func(addonStatus bool) string {
 	return "disabled"
 }
 
-var printAddonsList = func(cc *config.ClusterConfig) {
+var printAddonsList = func(cc *config.ClusterConfig, printDocs bool) {
 	addonNames := make([]string, 0, len(assets.Addons))
 	for addonName := range assets.Addons {
 		addonNames = append(addonNames, addonName)
 	}
 	sort.Strings(addonNames)
 
-	var tData [][]string
 	table := tablewriter.NewWriter(os.Stdout)
 	table.SetAutoFormatHeaders(true)
 	table.SetBorders(tablewriter.Border{Left: true, Top: true, Right: true, Bottom: true})
 	table.SetCenterSeparator("|")
-	if cc == nil {
-		table.SetHeader([]string{"Addon Name", "Maintainer"})
-	} else {
-		table.SetHeader([]string{"Addon Name", "Profile", "Status", "Maintainer"})
-	}
 
+	// Create table header
+	var tHeader []string
+	if cc == nil {
+		tHeader = []string{"Addon Name", "Maintainer"}
+	} else {
+		tHeader = []string{"Addon Name", "Profile", "Status", "Maintainer"}
+	}
+	if printDocs {
+		tHeader = append(tHeader, "Docs")
+	}
+	table.SetHeader(tHeader)
+
+	// Create table data
+	var tData [][]string
+	var temp []string
 	for _, addonName := range addonNames {
 		addonBundle := assets.Addons[addonName]
 		maintainer := addonBundle.Maintainer
 		if maintainer == "" {
-			maintainer = "unknown (third-party)"
+			maintainer = "3rd party (unknown)"
+		}
+		docs := addonBundle.Docs
+		if docs == "" {
+			docs = "n/a"
 		}
 		if cc == nil {
-			tData = append(tData, []string{addonName, maintainer})
-			continue
+			temp = []string{addonName, maintainer}
+		} else {
+			enabled := addonBundle.IsEnabled(cc)
+			temp = []string{addonName, cc.Name, fmt.Sprintf("%s %s", stringFromStatus(enabled), iconFromStatus(enabled)), maintainer}
 		}
-		enabled := addonBundle.IsEnabled(cc)
-		tData = append(tData, []string{addonName, cc.Name, fmt.Sprintf("%s %s", stringFromStatus(enabled), iconFromStatus(enabled)), maintainer})
+		if printDocs {
+			temp = append(temp, docs)
+		}
+		tData = append(tData, temp)
 	}
-
 	table.AppendBulk(tData)
+
 	table.Render()
 
 	v, _, err := config.ListProfiles()
