@@ -240,8 +240,13 @@ func waitForAPIServerProcess(cr command.Runner, start time.Time, timeout time.Du
 // dockerEnvCmd represents the docker-env command
 var dockerEnvCmd = &cobra.Command{
 	Use:   "docker-env",
-	Short: "Configure environment to use minikube's Docker daemon",
-	Long:  `Sets up docker env variables; similar to '$(docker-machine env)'.`,
+	Short: "Provides instructions to point your terminal's docker-cli to the Docker Engine inside minikube. (Useful for building docker images directly inside minikube)",
+	Long: `Provides instructions to point your terminal's docker-cli to the Docker Engine inside minikube. (Useful for building docker images directly inside minikube)
+
+For example, you can do all docker operations such as docker build, docker run, and docker ps directly on the docker inside minikube.
+
+Note: You need the docker-cli to be installed on your machine.
+docker-cli install instructions: https://minikube.sigs.k8s.io/docs/tutorials/docker_desktop_replacement/#steps`,
 	Run: func(cmd *cobra.Command, args []string) {
 		var err error
 
