@@ -54,8 +54,8 @@ type NetworkInfo struct {
 }
 
 // NewAddon creates a new Addon
-func NewAddon(assets []*BinAsset, enabled bool, addonName string, maintainer string, verifiedMaintainer string, docs string, images map[string]string, registries map[string]string) *Addon {
-	a := &Addon{
+func NewAddon(assets []*BinAsset, enabled bool, addonName, maintainer, verifiedMaintainer, docs string, images, registries map[string]string) *Addon {
+	return &Addon{
 		Assets:             assets,
 		enabled:            enabled,
 		addonName:          addonName,
@@ -65,10 +65,9 @@ func NewAddon(assets []*BinAsset, enabled bool, addonName string, maintainer str
 		Images:             images,
 		Registries:         registries,
 	}
-	return a
 }
 
-// Name get the addon name
+// Name gets the addon name
 func (a *Addon) Name() string {
 	return a.addonName
 }
@@ -140,7 +139,10 @@ var Addons = map[string]*Addon{
 	}, false, "dashboard", "Kubernetes", "", "https://minikube.sigs.k8s.io/docs/handbook/dashboard/", map[string]string{
 		"Dashboard":      "kubernetesui/dashboard:v2.6.0@sha256:4af9580485920635d888efe1eddbd67e12f9d5d84dba87100e93feb4e46636b3",
 		"MetricsScraper": "kubernetesui/metrics-scraper:v1.0.8@sha256:76049887f07a0476dc93efc2d3569b9529bf982b22d29f356092ce206e98765c",
-	}, nil),
+	}, map[string]string{
+		"Dashboard":      "docker.io",
+		"MetricsScraper": "docker.io",
+	}),
 	"default-storageclass": NewAddon([]*BinAsset{
 		MustBinAsset(addons.DefaultStorageClassAssets,
 			"storageclass/storageclass.yaml.tmpl",
@@ -191,7 +193,11 @@ var Addons = map[string]*Addon{
 		"Heketi":                 "heketi/heketi:10@sha256:76d5a6a3b7cf083d1e99efa1c15abedbc5c8b73bef3ade299ce9a4c16c9660f8",
 		"GlusterfileProvisioner": "gluster/glusterfile-provisioner:latest@sha256:9961a35cb3f06701958e202324141c30024b195579e5eb1704599659ddea5223",
 		"GlusterfsServer":        "gluster/gluster-centos:latest@sha256:8167034b9abf2d16581f3f4571507ce7d716fb58b927d7627ef72264f802e908",
-	}, nil),
+	}, map[string]string{
+		"Heketi":                 "docker.io",
+		"GlusterfsServer":        "docker.io",
+		"GlusterfileProvisioner": "docker.io",
+	}),
 	"efk": NewAddon([]*BinAsset{
 		MustBinAsset(addons.EfkAssets,
 			"efk/elasticsearch-rc.yaml.tmpl",
@@ -232,6 +238,7 @@ var Addons = map[string]*Addon{
 		"Elasticsearch":        "k8s.gcr.io",
 		"FluentdElasticsearch": "k8s.gcr.io",
 		"Kibana":               "docker.elastic.co",
+		"Alpine":               "docker.io",
 	}),
 	"ingress": NewAddon([]*BinAsset{
 		MustBinAsset(addons.IngressAssets,
@@ -243,11 +250,13 @@ var Addons = map[string]*Addon{
 		// https://github.com/kubernetes/ingress-nginx/blob/c32f9a43279425920c41ba2e54dfcb1a54c0daf7/deploy/static/provider/kind/deploy.yaml#L834
 		"IngressController": "ingress-nginx/controller:v1.2.1@sha256:5516d103a9c2ecc4f026efbd4b40662ce22dc1f824fb129ed121460aaa5c47f8",
 		// https://github.com/kubernetes/ingress-nginx/blob/fc38b9f2aa2d68ee00c417cf97e727b77a00c175/deploy/static/provider/kind/deploy.yaml#L621
-		"KubeWebhookCertgenCreate": "k8s.gcr.io/ingress-nginx/kube-webhook-certgen:v1.1.1@sha256:64d8c73dca984af206adf9d6d7e46aa550362b1d7a01f3a0a91b20cc67868660",
+		"KubeWebhookCertgenCreate": "ingress-nginx/kube-webhook-certgen:v1.1.1@sha256:64d8c73dca984af206adf9d6d7e46aa550362b1d7a01f3a0a91b20cc67868660",
 		// https://github.com/kubernetes/ingress-nginx/blob/fc38b9f2aa2d68ee00c417cf97e727b77a00c175/deploy/static/provider/kind/deploy.yaml#L673
-		"KubeWebhookCertgenPatch": "k8s.gcr.io/ingress-nginx/kube-webhook-certgen:v1.1.1@sha256:64d8c73dca984af206adf9d6d7e46aa550362b1d7a01f3a0a91b20cc67868660",
+		"KubeWebhookCertgenPatch": "ingress-nginx/kube-webhook-certgen:v1.1.1@sha256:64d8c73dca984af206adf9d6d7e46aa550362b1d7a01f3a0a91b20cc67868660",
 	}, map[string]string{
-		"IngressController": "k8s.gcr.io",
+		"IngressController":        "k8s.gcr.io",
+		"KubeWebhookCertgenCreate": "k8s.gcr.io",
+		"KubeWebhookCertgenPatch":  "k8s.gcr.io",
 	}),
 	"istio-provisioner": NewAddon([]*BinAsset{
 		MustBinAsset(addons.IstioProvisionerAssets,
@@ -257,7 +266,9 @@ var Addons = map[string]*Addon{
 			"0640"),
 	}, false, "istio-provisioner", "3rd party (Istio)", "", "https://istio.io/latest/docs/setup/platform-setup/minikube/", map[string]string{
 		"IstioOperator": "istio/operator:1.12.2@sha256:42c7609872882cb88728a1592561b4046dac6d05b6002cbdc815b84c86a24f08",
-	}, nil),
+	}, map[string]string{
+		"IstioOperator": "docker.io",
+	}),
 	"istio": NewAddon([]*BinAsset{
 		MustBinAsset(addons.IstioAssets,
 			"istio/istio-default-profile.yaml.tmpl",
@@ -274,7 +285,10 @@ var Addons = map[string]*Addon{
 	}, false, "kong", "3rd party (Kong HQ)", "", "https://minikube.sigs.k8s.io/docs/handbook/addons/kong-ingress/", map[string]string{
 		"Kong":        "kong:2.7@sha256:4d3e93207305ace881fe9e95ac27717b6fbdd9e0ec1873c34e94908a4f4c9335",
 		"KongIngress": "kong/kubernetes-ingress-controller:2.1.1@sha256:60e4102ab2da7f61e9c478747f0762d06a6166b5f300526b237ed7354c3cb4c8",
-	}, nil),
+	}, map[string]string{
+		"Kong":        "docker.io",
+		"KongIngress": "docker.io",
+	}),
 	"kubevirt": NewAddon([]*BinAsset{
 		MustBinAsset(addons.KubevirtAssets,
 			"kubevirt/pod.yaml.tmpl",
@@ -283,7 +297,9 @@ var Addons = map[string]*Addon{
 			"0640"),
 	}, false, "kubevirt", "3rd party (KubeVirt)", "", "https://minikube.sigs.k8s.io/docs/tutorials/kubevirt/", map[string]string{
 		"Kubectl": "bitnami/kubectl:1.17@sha256:de642e973d3d0ef60e4d0a1f92286a9fdae245535c5990d4762bbe86fcf95887",
-	}, nil),
+	}, map[string]string{
+		"Kubectl": "docker.io",
+	}),
 	"metrics-server": NewAddon([]*BinAsset{
 		MustBinAsset(addons.MetricsServerAssets,
 			"metrics-server/metrics-apiservice.yaml.tmpl",
@@ -346,10 +362,11 @@ var Addons = map[string]*Addon{
 			"registry-proxy.yaml",
 			"0640"),
 	}, false, "registry", "Google", "", "", map[string]string{
-		"Registry":          "registry:2.7.1@sha256:d5459fcb27aecc752520df4b492b08358a1912fcdfa454f7d2101d4b09991daa",
+		"Registry":          "registry:2.8.1@sha256:83bb78d7b28f1ac99c68133af32c93e9a1c149bcd3cb6e683a3ee56e312f1c96",
 		"KubeRegistryProxy": "google_containers/kube-registry-proxy:0.4@sha256:1040f25a5273de0d72c54865a8efd47e3292de9fb8e5353e3fa76736b854f2da",
 	}, map[string]string{
 		"KubeRegistryProxy": "gcr.io",
+		"Registry":          "docker.io",
 	}),
 	"registry-creds": NewAddon([]*BinAsset{
 		MustBinAsset(addons.RegistryCredsAssets,
@@ -359,7 +376,9 @@ var Addons = map[string]*Addon{
 			"0640"),
 	}, false, "registry-creds", "3rd party (UPMC Enterprises)", "", "https://minikube.sigs.k8s.io/docs/handbook/registry/", map[string]string{
 		"RegistryCreds": "upmcenterprises/registry-creds:1.10@sha256:93a633d4f2b76a1c66bf19c664dbddc56093a543de6d54320f19f585ccd7d605",
-	}, nil),
+	}, map[string]string{
+		"RegistryCreds": "docker.io",
+	}),
 	"registry-aliases": NewAddon([]*BinAsset{
 		MustBinAsset(addons.RegistryAliasesAssets,
 			"registry-aliases/registry-aliases-sa.tmpl",
@@ -393,6 +412,7 @@ var Addons = map[string]*Addon{
 	}, map[string]string{
 		"CoreDNSPatcher": "quay.io",
 		"Pause":          "gcr.io",
+		"Alpine":         "docker.io",
 	}),
 	"freshpod": NewAddon([]*BinAsset{
 		MustBinAsset(addons.FreshpodAssets,
@@ -442,7 +462,9 @@ var Addons = map[string]*Addon{
 			"0640"),
 	}, false, "logviewer", "3rd party (unknown)", "", "", map[string]string{
 		"LogViewer": "ivans3/minikube-log-viewer:latest@sha256:75854f45305cc47d17b04c6c588fa60777391761f951e3a34161ddf1f1b06405",
-	}, nil),
+	}, map[string]string{
+		"LogViewer": "docker.io",
+	}),
 	"gvisor": NewAddon([]*BinAsset{
 		MustBinAsset(addons.GvisorAssets,
 			"gvisor/gvisor-pod.yaml.tmpl",
@@ -512,7 +534,10 @@ var Addons = map[string]*Addon{
 	}, false, "metallb", "3rd party (MetalLB)", "", "", map[string]string{
 		"Speaker":    "metallb/speaker:v0.9.6@sha256:c66585a805bed1a3b829d8fb4a4aab9d87233497244ebff96f1b88f1e7f8f991",
 		"Controller": "metallb/controller:v0.9.6@sha256:fbfdb9d3f55976b0ee38f3309d83a4ca703efcf15d6ca7889cd8189142286502",
-	}, nil),
+	}, map[string]string{
+		"Speaker":    "docker.io",
+		"Controller": "docker.io",
+	}),
 	"ambassador": NewAddon([]*BinAsset{
 		MustBinAsset(addons.AmbassadorAssets,
 			"ambassador/ambassador-operator-crds.yaml.tmpl",
@@ -551,10 +576,11 @@ var Addons = map[string]*Addon{
 			"gcp-auth-webhook.yaml",
 			"0640"),
 	}, false, "gcp-auth", "Google", "", "https://minikube.sigs.k8s.io/docs/handbook/addons/gcp-auth/", map[string]string{
-		"KubeWebhookCertgen": "k8s.gcr.io/ingress-nginx/kube-webhook-certgen:v1.0@sha256:f3b6b39a6062328c095337b4cadcefd1612348fdd5190b1dcbcb9b9e90bd8068",
-		"GCPAuthWebhook":     "k8s-minikube/gcp-auth-webhook:v0.0.10@sha256:1ce1510da2a4af923e678d487ec0a78f4a8f2a65206a3aa8de659a196ae98d0f",
+		"KubeWebhookCertgen": "ingress-nginx/kube-webhook-certgen:v1.0@sha256:f3b6b39a6062328c095337b4cadcefd1612348fdd5190b1dcbcb9b9e90bd8068",
+		"GCPAuthWebhook":     "k8s-minikube/gcp-auth-webhook:v0.0.11@sha256:82efb346863dc47701586bebadd4cef998d4c6692d802ec3de68d451c87fb613",
 	}, map[string]string{
-		"GCPAuthWebhook": "gcr.io",
+		"GCPAuthWebhook":     "gcr.io",
+		"KubeWebhookCertgen": "k8s.gcr.io",
 	}),
 	"volumesnapshots": NewAddon([]*BinAsset{
 		// make sure the order of apply. `csi-hostpath-snapshotclass` must be the first position, because it depends on `snapshot.storage.k8s.io_volumesnapshotclasses`
@@ -689,7 +715,9 @@ var Addons = map[string]*Addon{
 			"0640"),
 	}, false, "portainer", "3rd party (Portainer.io)", "", "", map[string]string{
 		"Portainer": "portainer/portainer-ce:latest@sha256:4f126c5114b63e9d1bceb4b368944d14323329a9a0d4e7bb7eb53c9b7435d498",
-	}, nil),
+	}, map[string]string{
+		"Portainer": "docker.io",
+	}),
 	"inaccel": NewAddon([]*BinAsset{
 		MustBinAsset(addons.InAccelAssets,
 			"inaccel/fpga-operator.yaml.tmpl",
@@ -709,7 +737,7 @@ var Addons = map[string]*Addon{
 		MustBinAsset(addons.HeadlampAssets, "headlamp/headlamp-clusterrolebinding.yaml", vmpath.GuestAddonsDir, "headlamp-clusterrolebinding.yaml", "6040"),
 	}, false, "headlamp", "3rd party (kinvolk.io)", "", "https://minikube.sigs.k8s.io/docs/handbook/addons/headlamp/",
 		map[string]string{
-			"Headlamp": "kinvolk/headlamp:v0.9.0@sha256:465aaee6518f3fdd032965eccd6a8f49e924d144b1c86115bad613872672ec02",
+			"Headlamp": "kinvolk/headlamp:v0.11.1@sha256:2547c6f5d5186a2c01822648989d49d9853fecda14bca96a0bf4a0547ea1d613",
 		},
 		map[string]string{
 			"Headlamp": "ghcr.io",
@@ -733,36 +761,36 @@ func parseMapString(str string) map[string]string {
 	return mapResult
 }
 
-// mergeMaps creates a map with the union of `sourceMap` and `overrideMap` where collisions take the value of `overrideMap`.
-func mergeMaps(sourceMap, overrideMap map[string]string) map[string]string {
+// mergeMaps returns a map with the union of `source` and `override` where collisions take the value of `override`.
+func mergeMaps(source, override map[string]string) map[string]string {
 	result := make(map[string]string)
-	for name, value := range sourceMap {
-		result[name] = value
+	for k, v := range source {
+		result[k] = v
 	}
-	for name, value := range overrideMap {
-		result[name] = value
+	for k, v := range override {
+		result[k] = v
 	}
 	return result
 }
 
-// filterKeySpace creates a map of the values in `targetMap` where the keys are also in `keySpace`.
-func filterKeySpace(keySpace map[string]string, targetMap map[string]string) map[string]string {
+// filterKeySpace creates a map of the values in `target` where the keys are also in `keySpace`.
+func filterKeySpace(keySpace, target map[string]string) map[string]string {
 	result := make(map[string]string)
 	for name := range keySpace {
-		if value, ok := targetMap[name]; ok {
+		if value, ok := target[name]; ok {
 			result[name] = value
 		}
 	}
 	return result
 }
 
-// overrideDefaults creates a copy of `defaultMap` where `overrideMap` replaces any of its values that `overrideMap` contains.
-func overrideDefaults(defaultMap, overrideMap map[string]string) map[string]string {
-	return mergeMaps(defaultMap, filterKeySpace(defaultMap, overrideMap))
+// overrideDefaults creates a copy of `def` where `override` replaces any of its values that `override` contains.
+func overrideDefaults(def, override map[string]string) map[string]string {
+	return mergeMaps(def, filterKeySpace(def, override))
 }
 
 // SelectAndPersistImages selects which images to use based on addon default images, previously persisted images, and newly requested images - which are then persisted for future enables.
-func SelectAndPersistImages(addon *Addon, cc *config.ClusterConfig) (images, customRegistries map[string]string, err error) {
+func SelectAndPersistImages(addon *Addon, cc *config.ClusterConfig) (images, customRegistries map[string]string, _ error) {
 	addonDefaultImages := addon.Images
 	if addonDefaultImages == nil {
 		addonDefaultImages = make(map[string]string)
@@ -786,7 +814,7 @@ func SelectAndPersistImages(addon *Addon, cc *config.ClusterConfig) (images, cus
 		// Use newly configured custom images.
 		images = overrideDefaults(addonDefaultImages, newImages)
 		// Store custom addon images to be written.
-		cc.CustomAddonImages = mergeMaps(cc.CustomAddonImages, images)
+		cc.CustomAddonImages = mergeMaps(cc.CustomAddonImages, newImages)
 	}
 
 	// Use previously configured custom registries.
@@ -808,57 +836,67 @@ func SelectAndPersistImages(addon *Addon, cc *config.ClusterConfig) (images, cus
 		cc.CustomAddonRegistries = mergeMaps(cc.CustomAddonRegistries, customRegistries)
 	}
 
-	err = nil
 	// If images or registries were specified, save the config afterward.
 	if viper.IsSet(config.AddonImages) || viper.IsSet(config.AddonRegistries) {
 		// Since these values are only set when a user enables an addon, it is safe to refer to the profile name.
-		err = config.Write(viper.GetString(config.ProfileName), cc)
 		// Whether err is nil or not we still return here.
+		return images, customRegistries, config.Write(viper.GetString(config.ProfileName), cc)
 	}
-	return images, customRegistries, err
+	return images, customRegistries, nil
 }
 
 // GenerateTemplateData generates template data for template assets
-func GenerateTemplateData(addon *Addon, cfg config.KubernetesConfig, netInfo NetworkInfo, images, customRegistries map[string]string, enable bool) interface{} {
-
+func GenerateTemplateData(addon *Addon, cc *config.ClusterConfig, netInfo NetworkInfo, images, customRegistries map[string]string, enable bool) interface{} {
+	cfg := cc.KubernetesConfig
 	a := runtime.GOARCH
 	// Some legacy docker images still need the -arch suffix
-	// for  less common architectures blank suffix for amd64
+	// for less common architectures blank suffix for amd64
 	ea := ""
 	if runtime.GOARCH != "amd64" {
 		ea = "-" + runtime.GOARCH
 	}
 
+	v, err := util.ParseKubernetesVersion(cfg.KubernetesVersion)
+	if err != nil {
+		return errors.Wrap(err, "parsing Kubernetes version")
+	}
+
 	opts := struct {
-		PreOneTwentyKubernetes bool
-		Arch                   string
-		ExoticArch             string
-		ImageRepository        string
-		LoadBalancerStartIP    string
-		LoadBalancerEndIP      string
-		CustomIngressCert      string
-		IngressAPIVersion      string
-		ContainerRuntime       string
-		RegistryAliases        string
-		Images                 map[string]string
-		Registries             map[string]string
-		CustomRegistries       map[string]string
-		NetworkInfo            map[string]string
+		KubernetesVersion       map[string]uint64
+		PreOneTwentyKubernetes  bool
+		Arch                    string
+		ExoticArch              string
+		ImageRepository         string
+		LoadBalancerStartIP     string
+		LoadBalancerEndIP       string
+		CustomIngressCert       string
+		IngressAPIVersion       string
+		ContainerRuntime        string
+		RegistryAliases         string
+		Images                  map[string]string
+		Registries              map[string]string
+		CustomRegistries        map[string]string
+		NetworkInfo             map[string]string
+		LegacyPodSecurityPolicy bool
+		LegacyRuntimeClass      bool
 	}{
-		PreOneTwentyKubernetes: false,
-		Arch:                   a,
-		ExoticArch:             ea,
-		ImageRepository:        cfg.ImageRepository,
-		LoadBalancerStartIP:    cfg.LoadBalancerStartIP,
-		LoadBalancerEndIP:      cfg.LoadBalancerEndIP,
-		CustomIngressCert:      cfg.CustomIngressCert,
-		RegistryAliases:        cfg.RegistryAliases,
-		IngressAPIVersion:      "v1", // api version for ingress (eg, "v1beta1"; defaults to "v1" for k8s 1.19+)
-		ContainerRuntime:       cfg.ContainerRuntime,
-		Images:                 images,
-		Registries:             addon.Registries,
-		CustomRegistries:       customRegistries,
-		NetworkInfo:            make(map[string]string),
+		KubernetesVersion:       make(map[string]uint64),
+		PreOneTwentyKubernetes:  false,
+		Arch:                    a,
+		ExoticArch:              ea,
+		ImageRepository:         cfg.ImageRepository,
+		LoadBalancerStartIP:     cfg.LoadBalancerStartIP,
+		LoadBalancerEndIP:       cfg.LoadBalancerEndIP,
+		CustomIngressCert:       cfg.CustomIngressCert,
+		RegistryAliases:         cfg.RegistryAliases,
+		IngressAPIVersion:       "v1", // api version for ingress (eg, "v1beta1"; defaults to "v1" for k8s 1.19+)
+		ContainerRuntime:        cfg.ContainerRuntime,
+		Images:                  images,
+		Registries:              addon.Registries,
+		CustomRegistries:        customRegistries,
+		NetworkInfo:             make(map[string]string),
+		LegacyPodSecurityPolicy: v.LT(semver.Version{Major: 1, Minor: 25}),
+		LegacyRuntimeClass:      v.LT(semver.Version{Major: 1, Minor: 25}),
 	}
 	if opts.ImageRepository != "" && !strings.HasSuffix(opts.ImageRepository, "/") {
 		opts.ImageRepository += "/"
@@ -869,10 +907,6 @@ func GenerateTemplateData(addon *Addon, cfg config.KubernetesConfig, netInfo Net
 
 	// maintain backwards compatibility with k8s < v1.19
 	// by using v1beta1 instead of v1 api version for ingress
-	v, err := util.ParseKubernetesVersion(cfg.KubernetesVersion)
-	if err != nil {
-		return errors.Wrap(err, "parsing Kubernetes version")
-	}
 	if semver.MustParseRange("<1.19.0")(v) {
 		opts.IngressAPIVersion = "v1beta1"
 	}
@@ -902,6 +936,15 @@ func GenerateTemplateData(addon *Addon, cfg config.KubernetesConfig, netInfo Net
 			opts.Registries[name] = "" // Avoid nil access when rendering
 		}
 
+		// tl;dr If the user specified a custom image remove the default registry
+		// Without the line below, if you try to overwrite an image the default registry is still used in the templating
+		// Example - image name: MetricsScraper, default registry: docker.io, default image: kubernetesui/metrics-scraper
+		// Passed on addon enable: --images=MetricsScraper=k8s.gcr.io/echoserver:1.4
+		// Without this line the resulting image would be docker.io/k8s.gcr.io/echoserver:1.4
+		if _, ok := cc.CustomAddonImages[name]; ok {
+			opts.Registries[name] = ""
+		}
+
 		if enable {
 			if override, ok := opts.CustomRegistries[name]; ok {
 				out.Infof("Using image {{.registry}}{{.image}}", out.V{
