@@ -276,7 +276,7 @@ func runImageList(ctx context.Context, t *testing.T, profile, testName, format,
 func expectedImageFormat(format string) []string {
 	return []string{
 		fmt.Sprintf(format, "k8s.gcr.io/pause"),
-		fmt.Sprintf(format, "k8s.gcr.io/kube-apiserver"),
+		fmt.Sprintf(format, "registry.k8s.io/kube-apiserver"),
 	}
 }
 
